@@ -2,7 +2,7 @@
 	dct64_altivec.c: Discrete Cosine Tansform (DCT) for Altivec
 
 	copyright ?-2006 by the mpg123 project - free software under the terms of the LGPL 2.1
-	see COPYING and AUTHORS files in distribution or http://mpg123.de
+	see COPYING and AUTHORS files in distribution or http://mpg123.org
 	initially written by Michael Hipp
 	altivec optimization by tmkk
 */
@@ -25,7 +25,7 @@
 #include <altivec.h>
 #endif
 
-void dct64(real *out0,real *out1,real *samples)
+void dct64_altivec(real *out0,real *out1,real *samples)
 {
   real __attribute__ ((aligned (16))) bufs[64];
 
